#include <initializer_list>
#include <iostream>
#include <algorithm>
#include "seznam.h"
using namespace std;

seznam::seznam()
{
	hlava = zarazka = new prvek(NULA, nullptr);
	aktualni = zarazka;
	cout << "konstruktor ()" << endl;
}

seznam::seznam(const seznam &s)
	: seznam()
{
	if (!s.prazdny())
	{
		operator+=(s);
		reset();
	}
	cout << "konstruktor (&)" << endl;
}

seznam::seznam(initializer_list<T> seznamek)
	:seznam()
{
	for (T pom : seznamek)
	{
		vloz_na_konec(pom);
	}
	reset();
	cout << "konstruktor {}" << endl;
}

seznam::seznam(seznam && s)
	: seznam()
{
	tunel(s);
	cout << "konstruktor (&&)" << endl;
}


seznam::~seznam()
{
	vyprazdni();
	delete hlava;
	cout << "destruktor" << endl;
}
void seznam::tunel(seznam& s)
{
	swap(hlava, s.hlava);
	swap(zarazka, s.zarazka);
	aktualni = hlava;
}

seznam &seznam::operator=(const seznam & s)
{
	vyprazdni();
	if (!s.prazdny())
	{
		operator+=(s);
	}
	cout << "operator = &" << endl;
	return *this;
}

seznam &seznam::operator=(seznam && s)
{
	//vyprazdni();
	tunel(s);
	cout << "operator = &&" << endl;
	return *this;
}

seznam & seznam::operator+=(const seznam & s)
{
	if (!s.prazdny())
	{
		prvek *pom = s.aktualni;
		s.reset();
		do
		{
			vloz_na_konec(s.aktualni_hodnota());
		} while (s.dalsi());
		s.aktualni = pom;
	}
	reset();
	return *this;
}

seznam seznam::operator+(const seznam &s) const
{
	seznam vysledek;
	vysledek += *this;
	vysledek += s;
	return vysledek;
}

void seznam::vyprazdni()
{
	if (prazdny()) return;
	for (prvek* pom = hlava; !prazdny(); pom = hlava)
	{
		hlava = pom->dalsi();
		delete pom;
	}
	aktualni = zarazka;
}

bool seznam::prazdny() const
{
	return hlava == zarazka;
}

void seznam::vloz(T co)
{
	if (prazdny())
	{
		hlava = new prvek(co, hlava);
		aktualni = hlava;
	}
	else
	{
		prvek *pom = new prvek(co, aktualni->dalsi());
		aktualni->dalsi(pom);
		aktualni = pom;
	}
}

void seznam::vloz_na_konec(T co)
{
	zarazka->data() = co;
	aktualni = zarazka;
	prvek *pom = new prvek(NULA);
	zarazka->dalsi(pom);
	zarazka = pom;
}

bool seznam::najdi(T co) const
{
	for (prvek *pom = hlava; pom != zarazka; pom = pom->dalsi())
	{
		if (pom->data() == co)
		{
			aktualni = pom;
			return true;
		}
	}
	return false;
}

T &seznam::aktualni_hodnota()
{
	if (prazdny())
	{
		throw out_of_range("prazdny seznam");
	}
	return aktualni->data();
}

T seznam::aktualni_hodnota() const
{
	if (prazdny())
	{
		throw out_of_range("prazdny seznam");
	}
	return aktualni->data();
}

bool seznam::dalsi() const
{
	if (prazdny() || posledni())
	{
		return false;
	}
	aktualni = aktualni->dalsi();
	return true;
}

bool seznam::posledni() const
{
	if (prazdny())
	{
		return false;
	}
	return aktualni->dalsi() == zarazka;
}

void seznam::reset() const
{
	if (!prazdny())
	{
		aktualni = hlava;
	}
}

void seznam::na_konec() const
{
	if (!prazdny())
	{
		while (dalsi())
			;
	}
}



T &seznam::prvek::data()
{
	return hodnota;
}

seznam::prvek * seznam::prvek::dalsi()
{
	return _dalsi;
}

void seznam::prvek::dalsi(prvek *dalsi)
{
	_dalsi = dalsi;
}

