#pragma once
#ifndef __MATICE_H__
#define __MATICE_H__
#include <iostream>
using namespace std;

template<typename T, int pocet>
class matice;

template<typename T, int pocet>
T stopa(matice<T, pocet>& m);

template<typename T = double, int pocet = 10>
class matice 
{
	T (*p)[pocet];                 // Ukazatel na pole
	static int pocet_matic;
public:
	//typedef T typ_prvku;               // Zverejnime typ prvku matice 
	using typ_prvku = T;
	matice(T d = 0.0);                 // Konstruktor
	matice(const matice&);             // Kopirovaci konstruktor
	matice(matice&&);                  // Tunelovaci konstruktor
	~matice();                         // Destruktor
	matice operator~() const;          // Operator transposice
	matice& operator=(const matice&);  // Kopirovaci prirazovaci operator
	matice& operator=(matice&&);       // Tunelovaci prirazovaci operator
	matice operator-() const;
	matice operator+(const matice& m) const;
	matice operator*(const matice& m) const;
	matice operator*(const T d) const;
	T* operator[](int) const;
	T& operator ()(int, int);
	T operator ()(int, int) const;
	static int kolik() { return pocet_matic; } // Vrati pocet instanci 
	void vypis();
	template<typename R> operator matice<R, pocet>() const;
	friend T stopa<T, pocet>(matice<T, pocet> &m);
	friend ostream& operator<< <T, pocet>(ostream&, matice<T, pocet>&);
};

template<typename T, int pocet>
template<typename R>
matice<T, pocet>::operator matice<R, pocet>() const
{
	matice<R, pocet> vysledek;
	for (int i = 0; i < pocet; i++)
	{
		for (int j = 0; j < pocet; j++)
		{
			vysledek(i, j) = (R)p[i][j];
		}
	}
	return vysledek;
}

template<typename T, int pocet>
T stopa(matice<T, pocet>& m)
{
	T vysledek = 0;
	for (int i = 0; i < pocet; i++)
	{
		vysledek += m.p[i][i];
	}
	return vysledek;
}

template<class T, int pocet> int matice<T, pocet>::pocet_matic = 0;

template<typename T, int pocet>
matice<T, pocet>::matice(T d)         // Konstruktor
	: p(new T[pocet][pocet])          // d je pocatecni hodnota
{                                     // diagonalnich prvku
	pocet_matic++;                    // Pri neuspechu alokace vznikne vyjimka
	for (int i = 0; i < pocet; i++)
	{
		for (int j = 0; j < pocet; j++)
		{
			p[i][j] = 0;
		}
	}
	if (d)
	{
		for (int i = 0; i < pocet; i++) p[i][i] = d;
	}
}

template<typename T, int pocet>
matice<T, pocet>::~matice()
{
	delete[] p;
	pocet_matic--;
}

template<typename T, int pocet>
matice<T, pocet>::matice(const matice<T, pocet>& m)       // Kopirovaci konstruktor
	: p(new T[pocet][pocet])
{
	pocet_matic++;
	memcpy(p, m.p, sizeof(T)*pocet*pocet);
}

template<typename T, int pocet>
matice<T, pocet>::matice(matice<T, pocet> &&m)
{
	pocet_matic++;
	p = m.p;
	m.p = nullptr;
}

template<typename T, int pocet>
matice<T, pocet> matice<T, pocet>::operator+(const matice<T, pocet>& m) const
{
	matice<T, pocet> vysledek(*this);                         // Lokalni kopie leveho operandu
	for (int i = 0; i < pocet; i++)
	{
		for (int j = 0; j < pocet; j++)
		{
			vysledek.p[i][j] += m.p[i][j];
		}
	}
	return vysledek;
}

template<typename T, int pocet>
inline matice<T, pocet> matice<T, pocet>::operator*(const matice & m) const
{
	matice<T, pocet> vysledek;
	for (int i = 0; i < pocet; i++)
	{
		for (int j = 0; j < pocet; j++)
		{
			for (int k = 0; k < pocet; k++)
			{
				vysledek.p[i][j] += p[i][k] * m.p[k][j];
			}
		}
	}
	return vysledek;
}

template<typename T, int pocet>
matice<T, pocet> matice<T, pocet>::operator*(const T d) const
{
	matice<T, pocet> vysledek(*this);                         // Lokalni kopie leveho operandu
	for (int i = 0; i < pocet; i++)
	{
		for (int j = 0; j < pocet; j++)
		{
			vysledek.p[i][j] *= d;
		}
	}
	return vysledek;
}

template<typename T, int pocet>
T * matice<T, pocet>::operator[](int i) const
{
	return p[i];
}

template<typename T, int pocet>
T matice<T, pocet>::operator()(int i, int j) const
{
	if ((i < 0) || (j < 0) || (i >= pocet) || (j >= pocet))
	{
		throw out_of_range("index mimo meze");
	}
	return p[i][j];
}

template<typename T, int pocet>
T& matice<T, pocet>::operator()(int i, int j)
{
	if ((i < 0) || (j < 0) || (i >= pocet) || (j >= pocet))
	{
		throw out_of_range("index mimo meze");
	}
	return p[i][j];
}

template<typename T, int pocet>
matice<T, pocet> matice<T, pocet>::operator~() const     // Operator transpozice
{
	matice<T, pocet> vysledek(*this);                   // Pomocna instance - kopie operandu
	for (int i = 0; i < pocet; i++)
	{
		for (int j = i + 1; j < pocet; j++)
		{                              // Vlastni transposice
			swap(vysledek.p[i][j], vysledek.p[j][i]);
		}
	}
	return vysledek;                          // Vrati pomocnou instanci
}

template<typename T, int pocet>
matice<T, pocet>& matice<T, pocet>::operator=(const matice<T, pocet>& m)
{
	for (int i = 0; i < pocet; i++)
	{
		for (int j = 0; j < pocet; j++)
		{
			p[i][j] = m.p[i][j];
		}
	}
	return *this;
}

template<typename T, int pocet>
matice<T, pocet> & matice<T, pocet>::operator=(matice<T, pocet> && m)
{
	swap(p, m.p);
	return *this;
}

template<typename T, int pocet>
matice<T, pocet> matice<T, pocet>::operator-() const
{
	matice<T, pocet> vysledek;
	for (int i = 0; i < pocet; i++)
	{
		for (int j = 0; j < pocet; j++)
		{
			vysledek.p[i][j] = -p[i][j];
		}
	}
	return vysledek;
}

template<typename T, int pocet>
void matice<T, pocet>::vypis()
{
	for (int i = 0; i < pocet; i++)
	{
		for (int j = 0; j < pocet; j++)
		{
			cout << p[i][j] << " ";
		}
		cout << endl;
	}
}



template<typename T, int pocet>
matice<T, pocet> operator*(T d, matice<T, pocet> m)
{
	return m * d;
}

template<typename T, int pocet>
ostream& operator<< (ostream& proud,  matice<T, pocet>& m)
{
	for (int i = 0; i < pocet; i++)
	{
		for (int j = 0; j < pocet; j++)
		{
			cout << m.p[i][j] << " ";
		}
		cout << endl;
	}
	return proud;
}


template<class T>
using matice3 = matice<T, 3>;

#endif


