#include <iostream>
#include <stdexcept>
#include <exception>
#include <string>
#include <fstream>

// prints the explanatory string of an exception. If the exception is nested,
// recurses to print the explanatory of the exception it holds
void vypis_vyjimku(const std::exception& e, int level = 0)
{
	//std::cerr << typeid(e).name() << std::endl;
	std::cerr << std::string(level, ' ') << "vyjimka: " << e.what() << '\n';
	try {
		std::rethrow_if_nested(e);
	}
	catch (const std::exception& e) {
		vypis_vyjimku(e, level + 1);
	}
	catch (...) {}
}

void otevri_soubor(const std::string& jmeno)
{
	try {
		std::ifstream soubor(jmeno);
		soubor.exceptions(std::ios_base::failbit);
	}
	catch (...) {
		std::throw_with_nested(std::runtime_error("Nepodarilo se otevrit soubor " + jmeno));
	}
}

void zpracuj()
{
	try {
		otevri_soubor("neexistuje.txt");
	}
	catch (...) {
		std::throw_with_nested(std::runtime_error("volani zpracuj() neuspelo"));
	}
}

int main()
{
	try {
		zpracuj();
	}
	catch (const std::exception& e) {
		vypis_vyjimku(e);
	}
}