#ifndef __SEZNAM_H__
#define __SEZNAM_H__
#include <iostream>
#include <type_traits>
using namespace std;

template<typename T> 
class seznam
{
	class prvek;
	prvek *hlava, *zarka;
	mutable prvek *aktuln;
	void tunel(seznam &s);
	static_assert(is_trivially_copyable<T>::value, "Typ ukldanch dat mus bt triviln koprovateln");
public:
	typedef T typ;
	seznam();
	~seznam();
	bool przdn() const;
	void vyprzdni();
	seznam(const seznam &);
	seznam(seznam&&);
	seznam(initializer_list<T> hodnoty);
	seznam &operator=(const seznam &s);
	seznam &operator=(seznam &&s);
	seznam & seznam::operator+=(const seznam & s);
	seznam operator+(const seznam &s) const;
	void vlo(T co);
	void vlo_na_konec(T co);
	bool najdi(T co) const;
	T &aktuln_hodnota() const;
	bool dal() const;
	bool posledn() const;
	void reset() const;
	void na_konec() const;
};

template<typename T>
bool seznam<T>::dal() const
{
	if (przdn() || posledn())
	{
		return false;
	}
	aktuln = aktuln->dal();
	return true;
}

template<typename T>
void seznam<T>::tunel(seznam<T>& s)
{
	swap(hlava, s.hlava);
	swap(zarka, s.zarka);
	aktuln = hlava;
}


template<typename T>
T &seznam<T>::aktuln_hodnota() const
{
	if (przdn())
	{
		throw out_of_range("przdn seznam");
	}
	return aktuln->data();
}

template<typename T>
void seznam<T>::vlo(T co)
{
	if (przdn())
	{
		hlava = new prvek(co, hlava);
		aktuln = hlava;
	}
	else
	{
		prvek *pom = new prvek(co, aktuln->dal());
		aktuln->dal(pom);
		aktuln = pom;
	}
}

template<typename T>
bool seznam<T>::najdi(T co) const
{
	for (prvek *pom = hlava; pom != zarka; pom = pom->dal())
	{
		if (pom->data() == co)
		{
			aktuln = pom;
			return true;
		}
	}
	return false;
}

template<typename T>
seznam<T> seznam<T>::operator+(const seznam<T> &s) const
{
	seznam<T> vsledek;
	vsledek += *this;
	vsledek += s;
	return vsledek;
}



template<typename T>
bool seznam<T>::posledn() const
{
	if (przdn())
	{
		return false;
	}
	return aktuln->dal() == zarka;
}

template<typename T>
void seznam<T>::reset() const
{
	if (!przdn())
	{
		aktuln = hlava;
	}
}

template<typename T>
void seznam<T>::na_konec() const
{
	if (!przdn())
	{
		while (dal())
			;
	}
}



template<typename T>
seznam<T>::seznam()
{
	hlava = zarka = new prvek(T(), nullptr);
	aktuln = zarka;
	cout << "konstruktor () " <<  endl;
}

template<typename T>
seznam<T>::seznam(const seznam<T> &s)
	: seznam<T>()
{
	if (!s.przdn())
	{
		operator+=(s);
		reset();
	}
	cout << "konstruktor (&)" << endl;
}

template<typename T>
seznam<T>::seznam(seznam<T> && s)
	: seznam<T>()
{
	tunel(s);
	cout << "konstruktor (&&)" << endl;
}


template<typename T>
seznam<T>::seznam(initializer_list<T> hodnoty)
	:seznam<T>()
{
	for (T pom : hodnoty)
	{
		vlo_na_konec(pom);
	}
	reset();
	cout << "konstruktor {}" << endl;
}


template<typename T>
seznam<T> &seznam<T>::operator+=(const seznam<T> & s)
{
	if (!s.przdn())
	{
		prvek *pom = s.aktuln;
		s.reset();
		do
		{
			vlo_na_konec(s.aktuln_hodnota());
		} while (s.dal());
		s.aktuln = pom;
	}
	reset();
	return *this;
}

template<typename T>
void seznam<T>::vlo_na_konec(T co)
{
	zarka->data() = co;
	aktuln = zarka;
	prvek *pom = new prvek(T());
	zarka->dal(pom);
	zarka = pom;
}

template<typename T>
seznam<T> &seznam<T>::operator=(const seznam<T> & s)
{
	vyprzdni();
	if (!s.przdn())
	{
		operator+=(s);
	}
	cout << "opertor = &" << endl;
	return *this;
}

template<typename T>
seznam<T> &seznam<T>::operator=(seznam<T> && s)
{
	//vyprzdni();
	tunel(s);
	cout << "opertor = &&" << endl;
	return *this;
}


template<typename T>
void seznam<T>::vyprzdni()
{
	if (przdn()) return;
	for (prvek* pom = hlava; !przdn(); pom = hlava)
	{
		hlava = pom->dal();
		delete pom;
	}
	aktuln = zarka;
}

template<typename T>
bool seznam<T>::przdn() const
{
	return hlava == zarka;
}

template<typename T>
seznam<T>::~seznam()
{
	vyprzdni();
	delete hlava;
	cout << "destruktor" << endl;
}


template<typename T>
class seznam<T>::prvek
{
	T hodnota;
	prvek *_dal;
public:
	prvek(T hodnota, prvek *dal = nullptr)
		: hodnota(hodnota), _dal(dal)
	{}
	T &data();
	void dal(prvek * dal);
	prvek *dal();
};

template<typename T>
typename seznam<T>::prvek
*seznam<T>::prvek::dal()
{
	return _dal;
}

template<typename T>
T &seznam<T>::prvek::data()
{
	return hodnota;
}

template<typename T>
void seznam<T>::prvek::dal(typename seznam<T>::prvek *dal)
{
	_dal = dal;
}

#endif