#pragma once
#ifndef __MATICE_H__
#define __MATICE_H__
#include <iostream>
using namespace std;

template<typename T, int poet>
class matice;

template<typename T, int poet>
T stopa(matice<T, poet>& m);

template<typename T = double, int poet = 10>
class matice 
{
	T (*p)[poet];                 // Ukazatel na pole
	static int poet_matic;
public:
	//typedef T typ_prvku;               // Zveejnme typ prvk matice 
	using typ_prvku = T;
	matice(T d = 0.0);                 // Konstruktor
	matice(const matice&);             // Koprovac konstruktor
	matice(matice&&);                  // Tunelovac konstruktor
	~matice();                         // Destruktor
	matice operator~() const;          // Opertor transposice
	matice& operator=(const matice&);  // Koprovac piazovac opertor
	matice& operator=(matice&&);       // Tunelovac piazovac opertor
	matice operator-() const;
	matice operator+(const matice& m) const;
	matice operator*(const matice& m) const;
	matice operator*(const T d) const;
	T* operator[](int) const;
	T& operator ()(int, int);
	T operator ()(int, int) const;
	static int kolik() { return poet_matic; } // Vrt poet instanc 
	void vpis();
	template<typename R> operator matice<R, poet>() const;
	friend T stopa<T, poet>(matice<T, poet> &m);
	friend ostream& operator<< <T, poet>(ostream&, matice<T, poet>&);
};

template<typename T, int poet>
template<typename R>
matice<T, poet>::operator matice<R, poet>() const
{
	matice<R, poet> vsledek;
	for (int i = 0; i < poet; i++)
	{
		for (int j = 0; j < poet; j++)
		{
			vsledek(i, j) = (R)p[i][j];
		}
	}
	return vsledek;
}

template<typename T, int poet>
T stopa(matice<T, poet>& m)
{
	T vsledek = 0;
	for (int i = 0; i < poet; i++)
	{
		vsledek += m.p[i][i];
	}
	return vsledek;
}

template<class T, int pocet> int matice<T, pocet>::poet_matic = 0;

template<typename T, int poet>
matice<T, poet>::matice(T d)         // Konstruktor
	: p(new T[poet][poet])          // d je poten hodnota
{                                     // diagonlnch prvk
	poet_matic++;                    // Pi nespchu alokace vznikne vjimka
	for (int i = 0; i < poet; i++)
	{
		for (int j = 0; j < poet; j++)
		{
			p[i][j] = 0;
		}
	}
	if (d)
	{
		for (int i = 0; i < poet; i++) p[i][i] = d;
	}
}

template<typename T, int poet>
matice<T, poet>::~matice()
{
	delete[] p;
	poet_matic--;
}

template<typename T, int poet>
matice<T, poet>::matice(const matice<T, poet>& m)       // Koprovac konstruktor
	: p(new T[poet][poet])
{
	poet_matic++;
	memcpy(p, m.p, sizeof(T)*poet*poet);
}

template<typename T, int poet>
matice<T, poet>::matice(matice<T, poet> &&m)
{
	poet_matic++;
	p = m.p;
	m.p = nullptr;
}

template<typename T, int poet>
matice<T, poet> matice<T, poet>::operator+(const matice<T, poet>& m) const
{
	matice<T, poet> vsledek(*this);                         // Lokln kopie levho operandu
	for (int i = 0; i < poet; i++)
	{
		for (int j = 0; j < poet; j++)
		{
			vsledek.p[i][j] += m.p[i][j];
		}
	}
	return vsledek;
}

template<typename T, int poet>
inline matice<T, poet> matice<T, poet>::operator*(const matice & m) const
{
	matice<T, poet> vsledek;
	for (int i = 0; i < poet; i++)
	{
		for (int j = 0; j < poet; j++)
		{
			for (int k = 0; k < poet; k++)
			{
				vsledek.p[i][j] += p[i][k] * m.p[k][j];
			}
		}
	}
	return vsledek;
}

template<typename T, int poet>
matice<T, poet> matice<T, poet>::operator*(const T d) const
{
	matice<T, poet> vsledek(*this);                         // Lokln kopie levho operandu
	for (int i = 0; i < poet; i++)
	{
		for (int j = 0; j < poet; j++)
		{
			vsledek.p[i][j] *= d;
		}
	}
	return vsledek;
}

template<typename T, int poet>
T * matice<T, poet>::operator[](int i) const
{
	return p[i];
}

template<typename T, int poet>
T matice<T, poet>::operator()(int i, int j) const
{
	if ((i < 0) || (j < 0) || (i >= poet) || (j >= poet))
	{
		throw out_of_range("index mimo meze");
	}
	return p[i][j];
}

template<typename T, int poet>
T& matice<T, poet>::operator()(int i, int j)
{
	if ((i < 0) || (j < 0) || (i >= poet) || (j >= poet))
	{
		throw out_of_range("index mimo meze");
	}
	return p[i][j];
}

template<typename T, int poet>
matice<T, poet> matice<T, poet>::operator~() const     // Opertor transpozice
{
	matice<T, poet> vsledek(*this);                   // Pomocn instance - kopie operandu
	for (int i = 0; i < poet; i++)
	{
		for (int j = i + 1; j < poet; j++)
		{                              // Vlastn transposice
			swap(vsledek.p[i][j], vsledek.p[j][i]);
		}
	}
	return vsledek;                          // Vrt pomocnou instanci
}

template<typename T, int poet>
matice<T, poet>& matice<T, poet>::operator=(const matice<T, poet>& m)
{
	for (int i = 0; i < poet; i++)
	{
		for (int j = 0; j < poet; j++)
		{
			p[i][j] = m.p[i][j];
		}
	}
	return *this;
}

template<typename T, int poet>
matice<T, poet> & matice<T, poet>::operator=(matice<T, poet> && m)
{
	swap(p, m.p);
	return *this;
}

template<typename T, int poet>
matice<T, poet> matice<T, poet>::operator-() const
{
	matice<T, poet> vsledek;
	for (int i = 0; i < poet; i++)
	{
		for (int j = 0; j < poet; j++)
		{
			vsledek.p[i][j] = -p[i][j];
		}
	}
	return vsledek;
}

template<typename T, int poet>
void matice<T, poet>::vpis()
{
	for (int i = 0; i < poet; i++)
	{
		for (int j = 0; j < poet; j++)
		{
			cout << p[i][j] << " ";
		}
		cout << endl;
	}
}



template<typename T, int poet>
matice<T, poet> operator*(T d, matice<T, poet> m)
{
	return m * d;
}

template<typename T, int poet>
ostream& operator<< (ostream& proud,  matice<T, poet>& m)
{
	for (int i = 0; i < poet; i++)
	{
		for (int j = 0; j < poet; j++)
		{
			cout << m.p[i][j] << " ";
		}
		cout << endl;
	}
	return proud;
}


template<class T>
using matice3 = matice<T, 3>;

#endif


