#include <initializer_list>
#include <iostream>
#include <algorithm>
#include "seznam.h"
#include "chyba_seznamu.h"

using namespace std;

seznam::seznam()
{
	hlava = zarka = new prvek(NULA, nullptr);
	aktuln = zarka;
	cout << "konstruktor ()" << endl;
}

seznam::seznam(const seznam &s)
	: seznam()
{
	if (!s.przdn())
	{
		operator+=(s);
		reset();
	}
	cout << "konstruktor (&)" << endl;
}

seznam::seznam(initializer_list<T> seznmek)
	:seznam()
{
	for (T pom : seznmek)
	{
		vlo_na_konec(pom);
	}
	reset();
	cout << "konstruktor {}" << endl;
}

seznam::seznam(seznam && s)
	: seznam()
{
	tunel(s);
	cout << "konstruktor (&&)" << endl;
}


seznam::~seznam()
{
	vyprzdni();
	delete hlava;
	cout << "destruktor" << endl;
}
void seznam::tunel(seznam& s)
{
	swap(hlava, s.hlava);
	swap(zarka, s.zarka);
	aktuln = hlava;
}

seznam &seznam::operator=(const seznam & s)
{
	vyprzdni();
	if (!s.przdn())
	{
		operator+=(s);
	}
	cout << "opertor = &" << endl;
	return *this;
}

seznam &seznam::operator=(seznam && s)
{
	//vyprzdni();
	tunel(s);
	cout << "opertor = &&" << endl;
	return *this;
}

seznam & seznam::operator+=(const seznam & s)
{
	if (!s.przdn())
	{
		prvek *pom = s.aktuln;
		s.reset();
		do
		{
			vlo_na_konec(s.aktuln_hodnota());
		} while (s.dal());
		s.aktuln = pom;
	}
	reset();
	return *this;
}

seznam seznam::operator+(const seznam &s) const
{
	seznam vsledek;
	vsledek += *this;
	vsledek += s;
	return vsledek;
}

void seznam::vyprzdni()
{
	if (przdn()) return;
	for (prvek* pom = hlava; !przdn(); pom = hlava)
	{
		hlava = pom->dal();
		delete pom;
	}
	aktuln = zarka;
}

bool seznam::przdn() const
{
	return hlava == zarka;
}

void seznam::vlo(T co)
{
	if (przdn())
	{
		hlava = new prvek(co, hlava);
		aktuln = hlava;
	}
	else
	{
		prvek *pom = new prvek(co, aktuln->dal());
		aktuln->dal(pom);
		aktuln = pom;
	}
}

void seznam::vlo_na_konec(T co)
{
	zarka->data() = co;
	aktuln = zarka;
	prvek *pom = new prvek(NULA);
	zarka->dal(pom);
	zarka = pom;
}

bool seznam::najdi(T co) const
{
	for (prvek *pom = hlava; pom != zarka; pom = pom->dal())
	{
		if (pom->data() == co)
		{
			aktuln = pom;
			return true;
		}
	}
	return false;
}

T &seznam::aktuln_hodnota()
{
	if (przdn())
	{
		throw chyba_seznamu("przdn seznam");
	}
	return aktuln->data();
}

T seznam::aktuln_hodnota() const
{
	if (przdn())
	{
		throw chyba_seznamu("przdn seznam");
	}
	return aktuln->data();
}

bool seznam::dal() const
{
	if (przdn() || posledn())
	{
		return false;
	}
	aktuln = aktuln->dal();
	return true;
}

bool seznam::posledn() const
{
	if (przdn())
	{
		return false;
	}
	return aktuln->dal() == zarka;
}

void seznam::reset() const
{
	if (!przdn())
	{
		aktuln = hlava;
	}
}

void seznam::na_konec() const
{
	if (!przdn())
	{
		while (dal())
			;
	}
}



T &seznam::prvek::data()
{
	return hodnota;
}

seznam::prvek * seznam::prvek::dal()
{
	return _dal;
}

void seznam::prvek::dal(prvek *dal)
{
	_dal = dal;
}

